<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version   $Id$
 * @author 	Marcelo Mottalli <marcelo@gforgegroup.com>
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
 
require_once('initialize.php');
require_once('MailParser.php');
require_once('session/Session.php');
require_once('core/User.php');
require_once('plugin/ActionManager.php');

$session = Session::getSession();

$mail = new MailParser('php://stdin');

$email_from = strtolower($mail->getFrom());
$email_body = $mail->getBody();
$email_subject = $mail->getSubject();

try {
	// Get the user with the given email address
	$users = UserPeer::retrieveByEmail($email_from);
	if (count($users) > 1) {
		throw new Exception(Language::getSessText('Gateway.TooManyUsers', array($email_from)));
	} elseif (count($users) == 0) {
		throw new Exception(Language::getSessText('Gateway.NoUser', array($email_from)));
	}
	
	$user = $users[0];
	// Start a session as the user that sent the email
	$session->setUser($user);
	
	// Get the section and ref_id from subject
	if (!preg_match('/(.*) ?\\(([[:alnum:]]*)-([0-9]*)\\)$/',$email_subject,$matches)) {
		throw new Exception(Language::getSessText('Gateway.InvalidFormat'));
	}
	
	// Some UTF8 messages are base64 encoded
	if (stripos($mail->getHeader('Content-Transfer-Encoding'), 'base64') !== false) {
		$email_body = base64_decode($email_body);
		if ($email_body === false) {
			throw new Exception(Language::getSessText('Gateway.InvalidBodyFormat'));
		}
	}
	
	// Send the notification
	// note that $params['subject'] != $mail->getSubject() (the first one has the
	// section and ref_id stripped out)
	$params = array();
	$params['subject'] = $matches[1];
	$params['section'] = $matches[2];
	$params['ref_id'] = $matches[3];
	$params['mail_object'] = $mail;
	$params['from_user'] = $user;
	$params['body'] = $email_body;
	
	ActionManager::notify('MAIL_GATEWAY', $params);
} catch (Exception $ex) {
	$subject = Language::getSessText('Gateway.ErrorSubject');
	$message = Language::getSessText('Gateway.ErrorResponse', array($ex->getMessage(), $email_body));
	
	SystemUtils::sendMail($subject, $message, $email_from);
}
?>
