<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version   $Id: cleanup.php 489 2006-02-26 20:04:55Z tperdue $
 * @author	Marcelo Mottalli <mottalli@gforgegroup.com>
 * @date 2006-02-27
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

require_once('lib/core/UserPeer.php');
require_once('lib/core/ProjectPeer.php');

/**
 * This class has several functions for managing /etc/[passwd|group|shadow] files
 */
class SystemConfigManagerSolaris {

	private $global_users = array();
	private $gforge_users = array();
	private $gforge_users_aux = array();
	private $users = array();
	
	private $global_groups = array();
	private $gforge_groups = array();
	private $gforge_groups_aux = array();
	private $groups = array();
	
	private $testing;
	private $changed = true;
	
	public function __construct($testing = false) {
		$this->testing = $testing;
		$this->changed = false;
		$this->parseGroups();
		$this->parsePasswd();
	}
	
	public function purgeGForgeGroups() {
		foreach ($this->gforge_groups as $groupname => $foo) {
			unset($this->groups[$groupname]);
		}
		$this->gforge_groups = array();
	}
	
	public function purgeGForgeUsers() {
		foreach ($this->gforge_users as $username => $foo) {
			unset($this->users[$username]);
		}
		$this->gforge_users = array();
	}

	public function createGForgeGroup($groupname, $gid) {
		if ($this->groupExists($groupname)) return true;
		$group = array('name' => $groupname, 'password' => 'x', "gid" => $gid, "users" => array());
		$this->gforge_groups_aux[$groupname] = $group;
		$this->gforge_groups[$groupname] = $group;
		$this->groups[$groupname] = $group;
		$this->changed = true;
	}
	
	public function deleteGForgeGroup($groupname) {
		if (!array_key_exists($groupname, $this->gforge_groups) && !array_key_exists($groupname, $this->gforge_groups_aux)) {
			return false;
		} else {
			unset($this->gforge_groups_aux[$groupname]);
			unset($this->gforge_groups[$groupname]);
			unset($this->groups[$groupname]);
			$this->changed = true;
			return true;
		}
	}

	/**
	 * Creates/modifies information of a user. If the user already exists, it overwrites the
	 * information.
	 */
	public function createGForgeUser($username, $uid, $unix_password, $gid, $home, $shell, $gcos) {
		// Note: if the user already exists, the user's information is overwritten
		if ($username == 'root') return false;		// small security check
		
		$user = array('name' => $username, 'unix_password' => $unix_password, 'uid' => $uid,
			'gid' => $gid, 'gcos' => $gcos, 'home' => $home, 'shell' => $shell);
		
		if (array_key_exists($username, $this->gforge_users_aux)) {
			unset($this->gforge_users_aux[$username]);
		} elseif (array_key_exists($username, $this->users)) {
			unset($this->global_users[$username]);
		}
		
		$this->gforge_users_aux[$username] = $user;
		$this->gforge_users[$username] = $user;
		$this->users[$username] = $user;
		
		$this->changed = true;
		return true;
	}
	
	public function userExists($username) {
		$res = array_key_exists($username, $this->users);
		if ($res === true && array_key_exists($username, $this->gforge_users_aux)) {
			$this->gforge_users[$username] = $this->users[$username];
		}
		return $res;
	}
	
	public function groupExists($groupname) {
		$res = array_key_exists($groupname, $this->groups);
		if ($res === true && array_key_exists($groupname, $this->gforge_groups_aux)) {
			$this->gforge_groups[$groupname] = $this->groups[$groupname];
		}
		return $res;
	}
	
	public function addUserToGroup($username, $groupname) {
		if (!$this->groupExists($groupname)) return false;
		elseif ($this->isUserInGroup($username, $groupname)) return true;	// user already in group
		
		if (array_key_exists($groupname, $this->gforge_groups)) {
			$this->gforge_groups[$groupname]['users'][] = $username;
		} elseif (array_key_exists($groupname, $this->global_groups)) {
			$this->global_groups[$groupname]['users'][] = $username;
		}
		
		$this->groups[$groupname]['users'][] = $username;
		$this->changed = true;

		return true;
	}
	
	public function deleteGForgeUser($username) {
		if (!array_key_exists($username, $this->gforge_users)) {
			return false;
		}
		
		foreach ($this->gforge_groups as $groupname => $foo) {
			$this->deleteUserFromGroup($username, $groupname);
		}
		
		unset($this->gforge_users[$username]);
		unset($this->users[$username]);
		$this->changed = true;
		
		return true;
	}
	
	public function deleteUserFromGroup($username, $groupname) {
		if (!$this->isUserInGroup($username, $groupname)) {
			return false;
		}
		
		unset($this->groups[$groupname]['users'][$username]);
		if (array_key_exists($username, $this->gforge_groups[$groupname]['users'])) {
			unset($this->gforge_groups[$groupname]['users'][$username]);
		}
		
		$this->changed = true;
		return true;
	}

	public function isUserInGroup($username, $groupname) {
		if (!$this->groupExists($groupname)) return false;
		return (array_search($username, $this->groups[$groupname]['users']) !== false);
	}
	
	/**
	 * Writes all the changes to the disk. This should be called when you finish making ALL your
	 * changes, and not after individual changes.
	 */
	public function writeChanges() {
		if (!$this->changed) return true;
		
		// First, write /etc/group
		if ($this->testing) $file = '/etc/group.gforge-debug';
		else $file = '/etc/group';
		
		$fh = fopen($file, 'w');
		if (!$fh) {
			throw new Exception('Couldn\'t open '.$file.' for writing');
		} else {
			$this->writeGroups($fh, $this->global_groups);
			$this->writeGroups($fh, $this->gforge_groups);
		}
		fclose ($fh);
		
		// write /etc/passwd
		if ($this->testing) $file = '/etc/passwd.gforge-debug';
		else $file = '/etc/passwd';
		$fh = fopen($file, 'w');
		if (!$fh) {
			throw new Exception('Couldn\'t open '.$file.' for writing');
		} else {
			$this->writePasswd($fh, $this->global_users);
			$this->writePasswd($fh, $this->gforge_users);
		}
		fclose ($fh);
		$output = array();
		SystemUtils::execute('pwconv', $output);
	}
	
	private function writeGroups($fh, $groups) {
		foreach ($groups as $groupname => $group) {
			$line = $group['name'].':'.$group['password'].':'.$group['gid'].':'.implode(',', $group['users']);
			fputs($fh, $line."\n");
		}
	}
	
	private function writePasswd($fh, $users) {
		foreach ($users as $useranme => $user) {
			$line = $user['name'].':'.$user['unix_password'].':'.$user['uid'].':'.$user['gid'].':'.$user['gcos'].':'.
				$user['home'].':'.$user['shell'];
			fputs($fh, $line."\n");
		}
	}
	
	private function parseGroups() {
		$groups = @file('/etc/group');
		if ($groups !== false) {
			$n = count($groups);
			for ($i=0; $i < $n; $i++) {
				$group_line = $groups[$i];
				
				$group_line = trim($group_line);
				if (empty($group_line)) continue;
	
				list($name, $password, $gid, $users) = explode(':', $group_line);
				$Project = ProjectPeer::retrieveByUnixName($name);

				if (empty($users)) {
					$users = array();
				} else {
					$users = split(', *', $users);
				}
				$group = array("name" => $name, "password" => $password, "gid" => $gid, "users" => $users);
				
				if (is_object($Project) && $gid == $Project->getProjectId() + 50000) {
					$this->gforge_groups_aux[$name] = $group;
				} else {
					$this->global_groups[$name] = $group;
				}
				
				$this->groups[] = $group;
			}
		}
	}
	
	private function parsePasswd() {
		$users = @file('/etc/passwd');
		if ($users !== false) {
			$n = count($users);
			for ($i=0; $i < $n; $i++) {
				$user_line = $users[$i];
				
				$user_line = trim($user_line);
				if (empty($user_line)) continue;

				list($name, $password, $uid, $gid, $gcos, $home, $shell) = explode(':', $user_line);
				$UserObj = UserPeer::retrieveByUnixName($name);
			
				$user = array("name" => $name, "unix_password" => $password, "uid" => $uid, 
						"gid" => $gid, "gcos" => $gcos, "home" => $home, "shell" => $shell);
				
				if (is_object($UserObj) && $uid == $UserObj->getUserId() + 20000) {
					$this->gforge_users_aux[$name] = $user;
				} else {
					$this->global_users[$name] = $user;
				}
				
				$this->users[] = $user;
			}
		}
	}
}
?>
