<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version   $Id$
 * @author Tim Perdue tim@gforge.org
 * @date 2006-07-20
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
require_once 'plugins/docman/lib/om/BaseDocmanFileVersion.php';
require_once("exceptions/PermissionDeniedException.php");
require_once("GFPermission.interface.php");
require_once("plugin/ActionManager.php");

/**
 * The skeleton for this class was autogenerated by Propel  * You should add additional methods to this class to meet the
 * application requirements.  This class will only be generated as
 * long as it does not already exist in the output directory.
 *
 * @package plugins.docman.lib
 */
class DocmanFileVersion extends BaseDocmanFileVersion implements GFPermission {
	const SECTION='docmanfileversion';
		
	/**
	 * isPublic - true/false
	 * @return boolean
	 */
	function isPublic() {
		return $this->getIsPublic();
	}

	/**
	 * Specifies if the current user can create an entry in the database for this object
	 *
	 * @return bool
	 * @todo Program the logic
	 */
	public function userCanCreate() {
		return $this->getDocmanFile()->userCanCreate();
	}

	/**
	 * If this person is an admin for this object
	 * @return boolean
	 */
	private function userIsAdmin() {
		return $this->getDocmanFile()->userIsAdmin();
	}

	/**
	 * Specifies if the current user can update this object's entry in the database
	 *
	 * @return bool
	 * @todo Program the logic
	 */
	public function userCanUpdate() {
		return $this->getDocmanFile()->userCanUpdate();
	}
	
	/**
	 * Specifies if the current user can delete this object's entry from the database
	 *
	 * @return bool
	 * @todo Program the logic
	 */
	public function userCanDelete() {
		return $this->getDocmanFile()->userCanDelete();
	}

	/**
	 * Specifies if the current user has permissions to access this object
	 *
	 * @return bool
	 * @todo Program the logic
	 */
	public function userCanRead() {
		return $this->getDocmanFile()->userCanRead();
	}

	/**
	 * Removes this object from datastore and sets delete attribute. Informs the plugin manager.
	 * This method was added by propel_customizer.php.
	 * 
	 * @param Connection $con
	 * @return void
	 * @throws PropelException
	 * @see DocmanFileVersion::setDeleted()
	 * @see DocmanFileVersion::isDeleted()
	 */
	function delete($con = null) {
		if (!$this->userCanDelete()) {
			throw new PermissionDeniedException("Could not delete DocmanFileVersion object: permission denied");
		}
		
		// Audit trail
		$audit = new AuditTrail();
		$audit->setUser(Session::getSession()->getUser());
		$audit->setSection(Project::SECTION);
		$audit->setRefId($this->getDocmanFile()->getDocmanFolder()->getProjectId());
		$audit->setFieldName(self::SECTION);
		$audit->setNewValue('_deleted_');
		$audit->setOldValue($this->getFile()->getFileName().' (v'.$this->getVersionNumber().')');
		$audit->save();

		ActionManager::notify("DOCMAN_FILE_VERSION_DELETE", $this);
		parent::delete($con);		
	}

	/**
	 * Stores the object in the database using the Propel methods. Informs the plugin manager.
	 * This method was added by propel_customizer.php.
	 *
	 * @param Connection $con
	 * @return void
	 * @throws PropelException
	 */
	function save($con = null) {
		if ($this->isNew() && !$this->userCanCreate()) {
			throw new PermissionDeniedException("Could not create DocmanFileVersion object: permission denied");
		} else if (!$this->isNew() && !$this->userCanUpdate()) {
			throw new PermissionDeniedException("Could not update DocmanFileVersion object: permission denied");
		}

		if ($this->isNew()) {
			//calculate version number
			$dfvs=$this->getDocmanFile()->getDocmanFileVersions();
			$latest_version=$this->getDocmanFile()->getLatestVersion();
			if (is_object($latest_version)) {
				$version=$this->getDocmanFile()->getLatestVersion()->getVersionNumber()+1;
			} else {
				$version=1;
			}
			$this->setVersionNumber($version);
		}
		//if you're not a doc edit, put it into the queue
		$perm=$this->getDocmanFile()->getDocmanFolder()->getProject()->getPermission();
		$rootfolderid=$this->getDocmanFile()->getDocmanFolder()->getRootFolder()->getDocmanFolderId();
		$this->setIsApproved($perm->check(DocmanPlugin::SECTION,0,DocmanPlugin::ADMIN) || $perm->check(DocmanFolder::SECTION, $rootfolderid, DocmanFolder::PERMLEVEL_WRITE));
		$this->setCreatedBy(Session::getSession()->getUser()->getUserId());

		ActionManager::notify("DOCMAN_FILE_VERSION_SAVE_BEFORE", $this);
		parent::save($con);
		ActionManager::notify("DOCMAN_FILE_VERSION_SAVE", $this);

		if ($this->getVersionNumber() <> 1) {
			$DocmanFile = DocmanFilePeer::retrieveByPk($this->getDocmanFileId());
			$notification = new DocmanFileNotification($DocmanFile);
	
			// Notify there was a change in the file
			NotificationManager::getManager()->queue($notification);
		}
	}
	
	/**
	 * @return	Fileystem
	 */
	public function getFile() {
		$c = new Criteria();
		$c->add(FilesystemPeer::SECTION, self::SECTION);
		$c->add(FileSystemPeer::REF_ID, $this->getPrimaryKey());
		return FilesystemPeer::doSelectOne($c);
	}
}
