<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version   $Id$
 * @author Tim Perdue tim@gforge.org
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

require_once('lib/html/ProjectTemplate.php');
require_once('lib/plugin/PluginResult.php');
require_once('plugins/docman/lib/html/DocmanTemplate.php');

class DocmanAdminTemplate extends DocmanTemplate {

	/**
	 * Header - this subclasses header and just passes params and data that is specific
	 * to this template
	 * @param array	array of parameters
	 */
	function header($params) {
		$params['selectedbottomtab']='docman';
		$Lang = Session::getSession()->getLanguage();
				
		if(!isset($params['project']) || !is_object($params['project'])) {
			Throw new exception('Missing project object in docman template');		
		}
		if (!Session::getSession()->isLoggedIn()) {
			Throw new PermissionDeniedException();
		}
		if (!$params['project']->getPermission()->check(DocmanPlugin::SECTION,0,DocmanPlugin::ADMIN)) {
			Throw new PermissionDeniedException();
		} 
		if (!isset($params['title'])) {
			Throw new exception('Missing title in docman template'); 
		}
	
		$params['title']=' '.$Lang->getText('DocmanAdmin.TabString').' > '.$params['title'];
		
		
		$this->addBreadCrumb('<a href="'.GFUrl::getUrl()->projectUrl($params['project'],'','docman','admin').'">'.$Lang->getText('Tabs.Admin').'</a>');
		
		return parent::header($params);
	}

	/**
	 *      getPluginNavElements - this overrides parent class and adds its own
	 *      plugin listener
	 *      @return PluginResultSet
	 */
	function getPluginNavElements($params) {
		$prs = parent::getPluginNavElements($params);
		$tprs = ActionManager::Notify('GET_DOCMAN_ADMIN_NAV_ELEMENTS',null);
		foreach ($tprs as $pr) {
			$prs->addResult($pr);
		}
		return $prs;
	}

	/**
	 * getTemplateNavElements - this function can be overridden
	 * by subclasses so you can add navigation entries
	 * @return array - complex array of data.
	 */
	function getTemplateNavElements($params) {
		$pine = $this->getPluginNavElements($params);
		$url = GFurl::getUrl();
		$ret = array();
		foreach ($pine as $result) {
			$ret[]=$result->getResult();
		}
		if (!isset($params['project'])) {
			echo 'missing project param';
			return $ret;
		}
		$ret[] = array(
			'title'=> Language::getSessText('Docman.Docman'),
			'elements'=> array(
				array('link'=>'/gf/project/'.$params['project']->getUnixName().'/docman/',
					'description'=> Language::getSessText('Common.Browse')),array('link' => $url->projectURL($params['project'],'','docman','admin', ''),
				'description' => Language::getSessText('Tabs.Admin'))
				)
			);
			
		return $ret;
	}
}

?>