<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version $Id$
 * @author Daniel A. Perez	daniel@gforgegroup.com
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

require_once('lib/core/Filesystem.php');
require_once('lib/core/FilesystemPeer.php');
require_once('plugins/docman/lib/DocmanFile.php');
require_once('plugins/docman/lib/DocmanFilePeer.php');
require_once('plugins/docman/lib/DocmanFileVersion.php');
require_once('plugins/docman/lib/DocmanFileVersionPeer.php');
require_once('plugins/docman/lib/DocmanFolder.php');
require_once('plugins/docman/lib/DocmanFolderPeer.php');
require_once('lib/exceptions/PermissionDeniedException.php');
require_once('Validator.php');

$session = Session::getSession();
$Lang = $session->getLanguage();

$project = GFUrl::getUrl()->getProject();

$where=$_POST['subdir'];
$description=isset($_POST['description']) ? $_POST['description'] : "";
$changes=isset($_POST['changes'])? $_POST['changes'] : "";

if (!$session->isLoggedIn()) {
	throw new PermissionDeniedException();
}

$DocmanFolder = DocmanFolderPeer::retrieveByPK($where);
if (!is_object($DocmanFolder) || $DocmanFolder->getProjectId()!=$project->getProjectId()) {
	throw new PermissionDeniedException();
}

try {
	//doc is uploaded rather than edited inline
	if (isset($_REQUEST['action_type']) && $_REQUEST['action_type'] == DocmanFile::EDITED_UPLOAD) {
		$file = $_FILES['doc'];
		$error = $file['error'];
		$filename=$file['name'];
		if ($error > 0 && $error != 4) {
			$fileerrors = StringUtils::getFileUploadErrorString($error, $filename);
			throw new InvalidDataException($fileerrors);
		}
	} else {
		$filename = $_POST['filename'];
		if ($filename == "") {
			$errors['description'] = Language::getSessText('Docman.EmptyFilename');
			throw new InvalidDataException($errors);
		}
	}
	$con = Propel::getConnection(DocmanFolderPeer::DATABASE_NAME);
	$con->begin();
	$DocmanFile = new DocmanFile();
	//$DocmanFolder->setDocCount($DocmanFolder->getDocCount()+1);
	$DocmanFile->setDocmanFolderId($DocmanFolder->getDocmanFolderId());
	$DocmanFile->setDescription(Validator::sanitizeString($description));
	$DocmanFile->setIsLocked(false);
	$dfv = new DocmanFileVersion();
	$DocmanFile->addDocmanFileVersion($dfv);
	$dfv->setChanges(Validator::sanitizeString($changes));
	$DocmanFile->save();
	// create the file in the FS
	$newfile = new Filesystem();
	$newfile->setRefId($dfv->getPrimaryKey());
	$newfile->setSection(DocmanFileVersion::SECTION);
	$newfile->setPostedBy($session->getUser()->getUserId());
	if (isset($_REQUEST['action_type']) && $_REQUEST['action_type'] == DocmanFile::EDITED_UPLOAD) {
		$newfile->setFileType($file['type']);
		$newfile->setFileName($file['name']);
	} else {
		$newfile->setFileType('text/html');
		$newfile->setFileName(trim($filename).".htm");
	}
	$newfile->save();
	if (isset($_REQUEST['action_type']) && $_REQUEST['action_type'] == DocmanFile::EDITED_UPLOAD) {
		$newfile->setFileDataFromPath($file['tmp_name']);
	} else {
		$newfile->setFileDataFromString($_POST['message']);
	}
	$con->commit();
	if ($dfv->getIsApproved()) {
		$feedback = $Lang->getText('DocmanFile.AddFeedback');
	} else {
		$feedback = $Lang->getText('DocmanFile.AddQueuedFeedback');
	}
	if (isset($_REQUEST['redirect_to']) && $_REQUEST['redirect_to'] == 'admin') {
		$redirect_to = 'admin';
	} else {
		$redirect_to = '';
	}
	GFUrl::redirect(GFUrl::getUrl()->projectURL($project,'','docman',$redirect_to,array('subdir'=>$where,'feedback'=>$feedback)));

} catch (InvalidDataException $ex) {
	$errors=$ex->getErrors();
	$feedback = implode('<br />',$errors);
	include('DocmanFileAdd.php');
}

?>
