<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version   $Id$
 * @author Marcelo Mottalli <marcelo@gforgegroup.com>
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
require_once('MailParser.php');

class MailboxParser {
	private $mailbox;
	private $fh;
	private $nextMessageLines;
	private $sizeLimit = 0;

	public function __construct($mailbox) {
		$this->mailbox = $mailbox;
		$this->fh = fopen($mailbox, 'r');
		$this->nextMessageLines = array();
	}
	
	/**
	 * @return	MailParser
	 */
	public function nextMessage() {
		if (feof($this->fh)) {
			fclose($this->fh);
			// end of file?
			return false;
		}
		
		$message_lines = $this->nextMessageLines;
		$this->nextMessageLines = array();
		$first_line = true;
		
		$actual_size = 0;
		foreach ($message_lines as $line) {
			$actual_size += strlen($line);
		}

		while (true) {
			if (feof($this->fh)) {
				// end of file
				break;
			}
			
			$line = fgets($this->fh, 4096);
			// New message? (first line in current message always start with "From ")
			if (preg_match('/^From .+@.+/', $line) && !$first_line) {
				$this->nextMessageLines[] = $line;
				break;
			}
			
			$actual_size += strlen($line);
			if ($this->sizeLimit > 0 && $actual_size > $this->sizeLimit) {
				// Message exceeded the maximum length. Trim the email at $this->sizeLimit chars.
				$line = substr($line, 0, ($actual_size - $this->sizeLimit));
				$message_lines[] = $line;
				
				// Advance the pointer to the next message
				while (true) {
					if (feof($this->fh)) {
						break;
					}
					$line = fgets($this->fh, 4096);
					if (preg_match('/^From .+@.+/', $line) && !$first_line) {
						$this->nextMessageLines[] = $line;
						break;
					}
				}
				break;
			} else {
				$message_lines[] = $line;
				$first_line = false;
			}
		}
		
		return new MailParser($message_lines);
	}
	
	public function setSizeLimit($limit) {
		$this->sizeLimit = $limit;
	}
}

?>