<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version   $Id: aclcheck.php 816 2006-04-18 17:36:20Z marcelo $
 * @author Marcelo Mottalli	marcelo@gforgegroup.com
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
 
require_once('initialize.php');
require_once('GFConf.php');
require_once('core/Project.php');
require_once('plugins/scm/lib/ScmCommit.php');
require_once('plugins/scm/lib/ScmPreference.php');

if ($argc != 3) {
	echo 'Usage: '.basename(__FILE__).' <project UNIX name> <log message file>';
	exit(1);
}

// Get the project object
$project_name = $argv[1];
$project = ProjectPeer::retrieveByUnixName($project_name);
if (is_null($project)) {
	echo 'Invalid project name: '.$project_name."\n";
	exit(1);
}

ActionManager::notify("SCMCVS_PRECOMMIT", $project);

// Retrieve the SCM preferences for this project
$preferences = ScmPreferencePeer::retrieveByPk($project->getProjectId());
if (is_null($preferences)) {
	// No preferences specified... let it proceed
	exit(0);
}

if (!$preferences->getRequireTrackerItemId()) {
	// doesn't require a tracker item ID... let it proceed
	exit(0);
}

// In this point, the log file must include at least one tracker item ID

$log_file = $argv[2];
if (!file_exists($log_file)) {
	echo 'File with log message '.$log_file.' doesn\'t exist';
	exit(1);
}

$log = file_get_contents($log_file);

$invalid_ids = array();
if (!ScmCommit::checkTrackerItemIds($log, $project, $invalid_ids)) {
	if (count($invalid_ids)>0) {
		echo 'The following Tracker Item Ids are not valid for this Project or you are violating commit rules:'."\n";
		echo '[#'.implode('], [#', $invalid_ids)."]\n";
	} else {
		echo 'Commit not related to any Tracker Item.'."\n";
	}
	echo 'Please specify one or more valid Tracker Item Ids in the commit '."\n";
	echo 'log as "[#nnnn]". For example:'."\n";
	echo '$ cvs commit -m "Fixed [#2708]"'."\n";
	exit(1);
}

//print_r($matches);

exit(0);
?>
