<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version   $Id$
 * @author Marcelo Mottalli	<marcelo@gforgegroup.com>
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

require_once('lib/initialize.php');
require_once('system/Cronjob.php');
require_once('plugins/scmperforce/lib/PerforceManager.php');
require_once('plugins/scmperforce/lib/PerforceRepository.php');

class CreatePerforceRepositories extends Cronjob {
	public function __construct() {
		$this->jobname = basename(__FILE__);
		parent::__construct();
	}
	
	public function run() {
		// Get all projects that use the plugin
		$c = new Criteria();
		$c->add(PluginPeer::PLUGIN_NAME, 'scmperforce');
		$project_plugins = ProjectPluginPeer::doSelectJoinPlugin($c);
		
		foreach ($project_plugins as $project_plugin) {
			$project = $project_plugin->getProject(); // get the project
			$plugin = $project->getPluginByName('scmperforce');
			
			$repos = new PerforceRepository($project);
			
			if (is_null($plugin) || !$repos->isConfigured()) {
				// Perforce not configured for this project
				continue;
			}
			$project_scm = $repos->getScmConfiguration(); // get the scm configuration for this project
			
			try {
				if ($project->isActive() && $project_scm->getControlled()) {
					if (!$repos->isCreated()) {
						$repos->create();
					} else {
						$repos->updateRepository();
					}

				}
			} catch (PerforceException $e) {
				$this->logCron($e->getMessage());
			}
		}
		
		try {
			// This could throw an exception if we're over the licence quota
			$this->createAnonymousUser();
		} catch (PerforceException $e) {
			$this->logCron($e->getMessage());
		}
		
		// Remove the depots of deleted projects
		$this->cleanup();
	}
	
	/**
	 * Creates a user for anonymous access
	 */
	private function createAnonymousUser() {
		$file = tempnam('/tmp', 'gf_perforce');
		$config = new PerforceConfigFile($file);
		$config->setVar('User', 'anonymous');
		$config->setVar('password', 'anonymous');
		$config->setVar('fullname', 'Anonymous user');
		$config->setVar('email', 'anonymous@'.GFConf::get('system.hostName'));
		$config->save();

		$manager = new PerforceManager();
		$manager->setUser($file);

		@unlink($file);
	}

	/**
	 * Remove the depots associated to deleted projects
	 */
	public function cleanup() {
		$manager = new PerforceManager();
		$c = new Criteria();	
		$c->add(DeletedProjectPeer::ISDELETED, 0);	
		$deletedProjects = DeletedProjectPeer::doSelect($c);
		
		foreach ($deletedProjects as $deletedProject){
			$deletedUnixName = $deletedProject->getUnixGroupName();

			if (!$manager->depotExists($deletedUnixName)) {
				continue;
			}
			
			$manager->deleteDepot($deletedUnixName);
			$deletedProject->delete();
		}	
	}
}

$cronjob = new CreatePerforceRepositories();
$cronjob->run();



?>