<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version   $Id$
 * @author Marcelo Mottalli <marcelo@gforgegroup.com>
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

require_once('system/Cronjob.php');
require_once('SystemUtils.php');
require_once('core/CronHistory.php');
require_once('core/ProjectPlugin.php');
require_once('core/Plugin.php');
require_once('plugins/scmsvn/lib/ScmsvnPlugin.php');
require_once('plugins/scmsvn/lib/SVNRepository.php');
require_once('plugins/scm/lib/ScmServer.php');

/**
 * This array maps the SCM server IDs found on the scm_server table with their
 * mount locations on this machine. The path must point to the local path of the
 * server's SVNRoot (for example, /mnt/machine2/svnroot)
 */

/*
$GLOBALS['mountMap'] = array(
	1 => '/mnt/server1/svroot',
	2 => '/mnt/server2/svnroot'
	);
*/
$GLOBALS['mountMap'] = array(
	);


class CreateSymlinksCronjob extends Cronjob {
	public function __construct() {
		$this->jobname = basename(__FILE__);
		parent::__construct();
	}

	public function run() {
		// Only run if splitting SCM servers and this server has an assigned ID
		if (!GFConf::get('system.splitScmServers') || !GFConf::varExists('scmsvn.scmServerId')) {
			return;
		}
		
		// Get all the projects that use SVN
		$c = new Criteria();
		$c->add(PluginPeer::PLUGIN_NAME, ScmsvnPlugin::getName());
		$c->addJoin(ProjectPluginPeer::PLUGIN_ID, PluginPeer::PLUGIN_ID);
		$c->addJoin(ProjectPeer::PROJECT_ID, ProjectPluginPeer::PROJECT_ID);
		$project_scms = ProjectScmPeer::doSelectJoinProject($c);
		
		foreach ($project_scms as $project_scm) {
			$project = $project_scm->getProject();
			$scm_server_id = $project_scm->getScmServerId();
			$scm_server = ScmServerPeer::retrieveByPk($scm_server_id);
			
			// If the repository is on this server, skip it
			if ($scm_server_id == GFConf::get('scmsvn.scmServerId')) {
				continue;
			}
			
			if (!array_key_exists($scm_server_id, $GLOBALS['mountMap'])) {
				// The mount path for this server isn't set. Display a warning.
				$this->logCron('Could not create symbolic links for repository "'.$project->getUnixName().'": The mount path for the server "'.$scm_server->getHostname().'" (ID #'.$scm_server_id.') wasn\'t set');
				continue;
			}
			
			// Mount path of the remote server's repositories in this server
			$mountPath = $GLOBALS['mountMap'][$scm_server_id];
			if (!is_dir($mountPath)) {
				$this->logCron('Could not create symbolic links for repository "'.$project->getUnixName().'": The mount path for the server "'.$scm_server->getHostname().'" (ID #'.$scm_server_id.') points to an invalid directory ("'.$mountPath.'")');
				continue;
			}
			
			// Path where the repository is supposed to be
			$reposPath = SystemUtils::buildFilePath($mountPath, '/'.$project->getUnixName());
			if (!is_dir($reposPath)) {
				$this->logCron('Could not create symbolic links for repository "'.$project->getUnixName().'": Repository '.$reposPath.' doesn\'t exist');
				continue;
			}
			
			// Local path inside the SVN Root where link should be made
			$repos = new SVNRepository($project);
			$localLink = $repos->getRoot();
			if (is_link($localLink)) {
				// Link already exists, do nothing
				continue;
			} elseif (is_dir($localLink) || file_exists($localLink)) {
				// There's already a repository or file with the same name
				$this->logCron('Could not create symbolic links for repository "'.$project->getUnixName().'": A directory for the repository ('.$localLink.') already exists');
				continue;
			}
			
			// So far so good. Now we must create the link.
			$output = array();
			$error = SystemUtils::execute('ln -s '.$reposPath.' '.$localLink, $output);
			if ($error) {
				$this->logCron('Could not create symbolic links for repository "'.$project->getUnixName().'": '.implode('', $output));
			}
		}
	}
}
$cronjob = new CreateSymlinksCronjob();
$cronjob->run();
?>