<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version   $Id$
 * @author Ruben Gutierrez	ruben@gforgegroup.com
 * @date 2006-03-16
 *
 * SOAP plugin for GForge5.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
 
require_once('plugins/xmlcompatibility/lib/soap/artifact/ArtifactTypeTypes.php');
require_once('plugins/tracker/lib/TrackerPeer.php');
require_once('lib/core/ProjectPeer.php');

//
//	ArtifactType
//
$server->register(
	'getArtifactTypes',
	array('session_ser'=>'xsd:string','group_id'=>'xsd:int'),
	array('getArtifactTypesResponse'=>'tns:ArrayOfArtifactType'),
	$uri,
	$uri.'#getArtifactTypes','rpc','encoded'
);

//
//	getArtifactTypes
//
function getArtifactTypes($session_ser, $group_id) {
	try {
		$session = continueSession($session_ser);
		
		if ($group_id < 0) {
			$tracker_id = -1 * $group_id;
			$c = new Criteria();
			// Remove this line when full Task support is added to the plugin
			$c->add(TrackerPeer::TRACKER_ID, $tracker_id);
			//$c->add(TrackerPeer::DATATYPE, Tracker::DATATYPE_TRACKER);
			$c->addAscendingOrderByColumn(TrackerPeer::DATATYPE);
			$trackers = TrackerPeer::doSelect($c);
			if (count($trackers) > 0) {
				$Project = $trackers[0]->getProject();
				$group_id = $Project->getProjectId();
			}
		} else {
			$c = new Criteria();
			$c->add(TrackerPeer::PROJECT_ID, $group_id);
			// Remove this line when full Task support is added to the plugin
			//$c->add(TrackerPeer::DATATYPE, Tracker::DATATYPE_TRACKER);
			$c->addAscendingOrderByColumn(TrackerPeer::DATATYPE);
			$trackers = TrackerPeer::doSelect($c);
		}
		if ($trackers == null) {
			$trackers = array();
		}

		return trackersToSoap($trackers, $group_id);
	} catch (Exception $ex) {
		return new soap_fault('', 'getArtifactTypes',
							'Could Not Get Artifact Types: '.$ex->getMessage(),
							'Could Not Get Artifact Types: '.$ex->getMessage());
	}
}

function trackersToSoap($trackers, $project_id) {
	$Project = ProjectPeer::retrieveByPk($project_id);

	if ($Project == null || !is_object($Project)) {
		return array();
	}

	$perm = new Permission($Project);
	$return = array();
	foreach ($trackers as $tracker) {
		if ($perm->check(Tracker::SECTION, $tracker->getTrackerId(), Tracker::PERMLEVEL_TECH)) {
			// Get list of extra fields for this artifact
	
			$extra_fields = array();
			$c = new Criteria();
			$c->addAscendingOrderByColumn(TrackerExtraFieldPeer::FIELD_ORDER);
			$c->addDescendingOrderByColumn(TrackerExtraFieldPeer::TRACKER_EXTRA_FIELD_ID);
			
			$trackerExtraFields = $tracker->getTrackerExtraFields($c);
			foreach ($trackerExtraFields as $trackerExtraField) {
				$trackerExtraFieldElements = array();
				$trackerExtraFieldElements = $trackerExtraField->getTrackerExtraFieldElements();
	
				$available_values = array();
				for ($j=0; $j < count($trackerExtraFieldElements); $j++) {
					$available_values[$j]["element_id"] = $trackerExtraFieldElements[$j]->getElementId();
					$available_values[$j]["element_name"] = $trackerExtraFieldElements[$j]->getElementName();
					$available_values[$j]["status_id"] = $trackerExtraFieldElements[$j]->getStatusId();
				}
	
				$extra_fields[] = array(
					"extra_field_id"=> $trackerExtraField->getTrackerExtraFieldId(),
					"field_name"	=> $trackerExtraField->getFieldName(),
					"field_type"	=> $trackerExtraField->getFieldType(),
					"attribute1"	=> $trackerExtraField->getAttribute1(),
					"attribute2"	=> $trackerExtraField->getAttribute2(),
					"is_required"	=> ($trackerExtraField->getIsRequired()==1) ? '1' : '0',
					"alias"			=> $trackerExtraField->getAlias(),
					"available_values"    =>$available_values,
					"default_selected_id" =>'0'		//TODO (not implemented yet)
				);
			}
	
			$return[] = array(
				'group_artifact_id'=>$tracker->getTrackerId(),
				'group_id'         =>$tracker->getProjectId(),
				'name'             =>$tracker->getTrackerName(),
				'description'      =>$tracker->getDescription(),
				'is_public'        =>($tracker->getIsPublic()==1) ? '1' : '0',
				'allow_anon'       =>'0',
				'due_period'       =>$tracker->getDuePeriod(),
				'datatype'         =>$tracker->getDataType(),
				'status_timeout'   =>'1209600',
				'extra_fields'     =>$extra_fields,
				'custom_status_field'=>'0'
			);
		}
	}

	return $return;
}

?>
