<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version   $Id: Layout.php 270 2005-12-31 03:02:47Z tperdue $
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

// This code generates a SOAP API for handling CruisecontrolAssignee

require_once('plugins/build/lib/CruisecontrolAssignee.php');
require_once('plugins/build/lib/CruisecontrolAssigneePeer.php');
require_once('Validator.php');

/* ******************************************************************** */
/*                                 DATA TYPES                           */
/* ******************************************************************** */
$server->wsdl->addComplexType(
	'CruisecontrolAssignee',
	'complexType',
	'struct',
	'sequence',
	'',
	array(
		'cruisecontrol_id' => array('name'=>'cruisecontrol_id', 'type' => 'xsd:int'),
		'assignee' => array('name'=>'assignee', 'type' => 'xsd:int'),
	)
);


$server->wsdl->addComplexType(
	'ArrayOfCruisecontrolAssignee',
	'complexType',
	'array',
	'',
	'SOAP-ENC:Array',
	array(),
	array(array('ref' => 'SOAP-ENC:arrayType', 'wsdl:arrayType' => 'tns:CruisecontrolAssignee[]')),
	'tns:CruisecontrolAssignee'
);

/* ******************************************************************** */
/*                                 METHODS                              */
/* ******************************************************************** */

$server->register(
	'getCruisecontrolAssignees',
	array(
		'sessionHash'	=> 'xsd:string',
		'cruisecontrol_id' => 'xsd:int',
		'assignee' => 'xsd:int',
	), 
	array('getCruisecontrolAssignees' => 'tns:ArrayOfCruisecontrolAssignee'),
	$uri, 
	$uri.'#getCruisecontrolAssignees',
	'rpc',
	'encoded'
);


/**
 * SOAP method to retrieve a list of CruisecontrolAssignee
 * @param	$cruisecontrol_id	Filter by cruisecontrol_id
 * @param	$assignee	Filter by assignee
 */
function getCruisecontrolAssignees($sessionHash, $cruisecontrol_id, $assignee) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($sessionHash, $ip);

	$crit = new Criteria();
	
	if (!is_null($cruisecontrol_id) && $cruisecontrol_id > 0) {
		$crit->add(CruisecontrolAssigneePeer::CRUISECONTROL_ID, $cruisecontrol_id);
	}
	if (!is_null($assignee) && $assignee > 0) {
		$crit->add(CruisecontrolAssigneePeer::ASSIGNEE, $assignee);
	}
	
	$objs = CruisecontrolAssigneePeer::doSelect($crit);
	$res = array();
	foreach ($objs as $obj) {
		if ($obj->userCanRead()) {
			$res[] = CruisecontrolAssigneeToSOAP($obj);
		}
	}
	return $res;
}



$server->register(
	'addCruisecontrolAssignee',
	array(
		'sessionHash'	=> 'xsd:string',
		'cruisecontrol_id' => 'xsd:int',
		'assignee' => 'xsd:int',
	), 
	array('addCruisecontrolAssignee' => 'xsd:int'),
	$uri, 
	$uri.'#addCruisecontrolAssignee',
	'rpc',
	'encoded'
);


/**
 * SOAP method to add a CruisecontrolAssignee
 * @param	$cruisecontrol_id
 * @param	$assignee
 */
function addCruisecontrolAssignee($sessionHash, $cruisecontrol_id, $assignee) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($sessionHash, $ip);

	$obj = new CruisecontrolAssignee();
	
	$obj->setCruisecontrolId( Validator::sanitizeInt($cruisecontrol_id) );
	$obj->setAssignee( Validator::sanitizeInt($assignee) );
	try {
		$obj->save();
		return 1;
	} catch (Exception $e) {
		return new soap_fault('', 'addCruisecontrolAssignee','Permission Denied - Cannot add This Record',$e->getMessage());
	}
}

$server->register(
	'deleteCruisecontrolAssignee',
	array(
		'sessionHash'	=> 'xsd:string',
		'cruisecontrol_id' => 'xsd:int',
		'assignee' => 'xsd:int',
	), 
	array('deleteCruisecontrolAssignee' => 'xsd:int'),
	$uri, 
	$uri.'#deleteCruisecontrolAssignee',
	'rpc',
	'encoded'
);

/**
 * SOAP method to delete a single CruisecontrolAssignee
 * @param	$cruisecontrol_id
 * @param	$assignee
 */
function deleteCruisecontrolAssignee($sessionHash, $cruisecontrol_id, $assignee) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($sessionHash, $ip);

	$obj = CruisecontrolAssigneePeer::retrieveByPK($cruisecontrol_id, $assignee);
	if (is_null($obj)) {
		return new soap_fault('', 'deleteCruisecontrolAssignee','Permission Denied - Cannot Delete This Record','Permission Denied - Cannot View This Record');
	}
	try {
		$obj->delete();
	} catch (Exception $e) {
		return new soap_fault('', 'deleteCruisecontrolAssignee','Permission Denied - Cannot Delete This Record',$e->getMessage());
	}
	return 1;
}

function CruisecontrolAssigneeToSOAP(&$obj) {
	$ret = array();
	$ret['cruisecontrol_id'] = $obj->getCruisecontrolId();
	$ret['assignee'] = $obj->getAssignee();
	return $ret;
}
?>
