<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version   $Id: Layout.php 270 2005-12-31 03:02:47Z tperdue $
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

// This code generates a SOAP API for handling CruisecontrolBuildChangeUser

require_once('plugins/build/lib/CruisecontrolBuildChangeUser.php');
require_once('plugins/build/lib/CruisecontrolBuildChangeUserPeer.php');
require_once('Validator.php');

/* ******************************************************************** */
/*                                 DATA TYPES                           */
/* ******************************************************************** */
$server->wsdl->addComplexType(
	'CruisecontrolBuildChangeUser',
	'complexType',
	'struct',
	'sequence',
	'',
	array(
		'cruisecontrol_build_id' => array('name'=>'cruisecontrol_build_id', 'type' => 'xsd:int'),
		'user_id' => array('name'=>'user_id', 'type' => 'xsd:int'),
	)
);


$server->wsdl->addComplexType(
	'ArrayOfCruisecontrolBuildChangeUser',
	'complexType',
	'array',
	'',
	'SOAP-ENC:Array',
	array(),
	array(array('ref' => 'SOAP-ENC:arrayType', 'wsdl:arrayType' => 'tns:CruisecontrolBuildChangeUser[]')),
	'tns:CruisecontrolBuildChangeUser'
);

/* ******************************************************************** */
/*                                 METHODS                              */
/* ******************************************************************** */

$server->register(
	'getCruisecontrolBuildChangeUsers',
	array(
		'sessionHash'	=> 'xsd:string',
		'cruisecontrol_build_id' => 'xsd:int',
		'user_id' => 'xsd:int',
	), 
	array('getCruisecontrolBuildChangeUsers' => 'tns:ArrayOfCruisecontrolBuildChangeUser'),
	$uri, 
	$uri.'#getCruisecontrolBuildChangeUsers',
	'rpc',
	'encoded'
);


/**
 * SOAP method to retrieve a list of CruisecontrolBuildChangeUser
 * @param	$cruisecontrol_build_id	Filter by cruisecontrol_build_id
 * @param	$user_id	Filter by user_id
 */
function getCruisecontrolBuildChangeUsers($sessionHash, $cruisecontrol_build_id, $user_id) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($sessionHash, $ip);

	$crit = new Criteria();
	
	if (!is_null($cruisecontrol_build_id)) {
		$crit->add(CruisecontrolBuildChangeUserPeer::CRUISECONTROL_BUILD_ID, $cruisecontrol_build_id);
	}
	if (!is_null($user_id)) {
		$crit->add(CruisecontrolBuildChangeUserPeer::USER_ID, $user_id);
	}
	
	$objs = CruisecontrolBuildChangeUserPeer::doSelect($crit);
	$res = array();
	foreach ($objs as $obj) {
		if ($obj->userCanRead()) {
			$res[] = CruisecontrolBuildChangeUserToSOAP($obj);
		}
	}
	return $res;
}



$server->register(
	'addCruisecontrolBuildChangeUser',
	array(
		'sessionHash'	=> 'xsd:string',
		'cruisecontrol_build_id' => 'xsd:int',
		'user_id' => 'xsd:int',
	), 
	array('addCruisecontrolBuildChangeUser' => 'xsd:int'),
	$uri, 
	$uri.'#addCruisecontrolBuildChangeUser',
	'rpc',
	'encoded'
);


/**
 * SOAP method to add a CruisecontrolBuildChangeUser
 * @param	$cruisecontrol_build_id
 * @param	$user_id
 */
function addCruisecontrolBuildChangeUser($sessionHash, $cruisecontrol_build_id, $user_id) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($sessionHash, $ip);

	$obj = new CruisecontrolBuildChangeUser();
	
	$obj->setCruisecontrolBuildId( Validator::sanitizeInt($cruisecontrol_build_id) );
	$obj->setUserId( Validator::sanitizeInt($user_id) );
	try {
		$obj->save();
		return 1;
	} catch (Exception $e) {
		return new soap_fault('', 'addCruisecontrolBuildChangeUser','Permission Denied - Cannot add This Record',$e->getMessage());
	}
}

$server->register(
	'deleteCruisecontrolBuildChangeUser',
	array(
		'sessionHash'	=> 'xsd:string',
		'cruisecontrol_build_id' => 'xsd:int',
		'user_id' => 'xsd:int',
	), 
	array('deleteCruisecontrolBuildChangeUser' => 'xsd:int'),
	$uri, 
	$uri.'#deleteCruisecontrolBuildChangeUser',
	'rpc',
	'encoded'
);

/**
 * SOAP method to delete a single CruisecontrolBuildChangeUser
 * @param	$cruisecontrol_build_id
 * @param	$user_id
 */
function deleteCruisecontrolBuildChangeUser($sessionHash, $cruisecontrol_build_id, $user_id) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($sessionHash, $ip);

	$obj = CruisecontrolBuildChangeUserPeer::retrieveByPK($cruisecontrol_build_id, $user_id);
	if (is_null($obj)) {
		return new soap_fault('', 'deleteCruisecontrolBuildChangeUser','Permission Denied - Cannot Delete This Record','Permission Denied - Cannot View This Record');
	}
	try {
		$obj->delete();
	} catch (Exception $e) {
		return new soap_fault('', 'deleteCruisecontrolBuildChangeUser','Permission Denied - Cannot Delete This Record',$e->getMessage());
	}
	return 1;
}

function CruisecontrolBuildChangeUserToSOAP(&$obj) {
	$ret = array();
	$ret['cruisecontrol_build_id'] = $obj->getCruisecontrolBuildId();
	$ret['user_id'] = $obj->getUserId();
	return $ret;
}
?>
