#!/usr/bin/php -f
<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version   $Id$
 * @author Ruben Gutierrez ruben@gforgegroup.com
 * @date 2006-09-07
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

define ('GREEN', "\033[01;32m" );
define ('NORMAL', "\033[00m" );
define ('RED', "\033[01;31m" );

require_once('initialize.php');
require('lib/GFConf.php');


function upgrade() {
	
	// Where the PHP code will live
	$gforge_lib_dir = '/opt/gforge5';
	if (!is_dir($gforge_lib_dir)) {
		show(RED."Error: GForge5 folder doesn't exist.");
		die();
	}
	$gforge_etc_dir = getenv('GFORGE_ETC_DIR');
	if (empty($gforge_etc_dir)) {
		$gforge_etc_dir = '/etc/gforge';
	}
	if (!is_dir($gforge_etc_dir)) {
		show(RED."Error: GForge5 config folder doesn't exist.");
		die();
	}
	$gforge_var_dir = getenv('GFORGE_VAR_DIR');
	if (empty($gforge_var_dir)) {
		$gforge_var_dir = '/var/lib/gforge';
	}
	if (!is_dir($gforge_var_dir)) {
		show(RED."Error: GForge5 var folder doesn't exist.");
		die();
	}

	$sysconf = GFConf::getMainConfigFile();
	$current = GFConf::getConfigArray($sysconf);
	$new = GFConf::getConfigArray($gforge_lib_dir.'/conf/gforge.conf.example');


	show(' * Checking main configuration file...');
	foreach($new as $key => $val) {
		if (!isset($current[$key])) {
			show(" * * The config variable ".NORMAL.$key.GREEN." is missing");
			show(" * * Please, type new a value or press enter to use the default one.");
			show(" * * Default value: ".NORMAL.printVal($val));

			$newval = trim(fgets(STDIN));
			if (strlen($newval) == 0) {
				$newval = printVal($val);
			}
			addToConf($sysconf, $key, $val, $newval);
		}
	}
	
	show(' * Checking plugins configuration files...');

	// Copy the new config files
	$allPlugins = GFConf::getAllPluginsConfigs($gforge_lib_dir);
	foreach ($allPlugins as $plugin => $conf) {
		$new_conf = "$gforge_etc_dir/plugins/$plugin/$plugin.conf";
		$new_dir = "$gforge_etc_dir/plugins/$plugin/";
		if (!file_exists($new_conf)) {
			mkdir($new_dir);
			copy($conf, $new_conf);
		}
	}

	// Check if there's new config variables
	$plugins = GFConf::getPluginsConfigArray();
	foreach ($plugins as $plugin => $current_val) {
		$plugconf = "$gforge_etc_dir/plugins/$plugin/$plugin.conf";
		$new = GFConf::getConfigArray("$gforge_lib_dir/plugins/$plugin/conf/$plugin.conf");
		
		foreach($new as $key => $val) {
			if (!isset($current_val[$key])) {
				show(" * * The config variable ".NORMAL.$key.GREEN." for plugin ".NORMAL.$plugin.GREEN." is missing");
				show(" * * Please, type new a value or press enter to use the default one.");
				show(" * * Default value: ".NORMAL.printVal($val));
				$newval = trim(fgets(STDIN));
				if (strlen($newval) == 0) {
					$newval = printVal($val);
				}
				addToConf($plugconf, $key, $val, $newval);
			}
		}
	}
	
	show(' * Checking symbolic links...');
	if (!file_exists("$gforge_lib_dir/www/tabber.js")) {
		show(' * * Creating tabber link...');
		run("cd $gforge_lib_dir/www && ln -s ../lib/external/tabber.js tabber.js", true);
	}
	if (!file_exists("$gforge_lib_dir/www/fckeditor")) {
		show(' * * Creating fckeditor link...');
		run("cd $gforge_lib_dir/www && ln -s ../lib/external/fckeditor fckeditor", true);
	}
	if (!file_exists("$gforge_lib_dir/www/datetimepicker.js")) {
		show(' * * Creating datetimepicker link...');
		run("cd $gforge_lib_dir/www && ln -s ../lib/external/datetimepicker.js datetimepicker.js", true);
	}
	if (!file_exists("$gforge_lib_dir/www/prototype.js")) {
		show(' * * Creating prototype framework link...');
		run("cd $gforge_lib_dir/www && ln -s ../lib/external/prototype.js prototype.js", true);
	}
	if (!file_exists("$gforge_lib_dir/www/scriptaculous.js")) {
		show(' * * Creating scriptaculous link...');
		run("cd $gforge_lib_dir/www && ln -s ../lib/external/scriptaculous.js scriptaculous.js", true);
	}
	if (!file_exists("$gforge_lib_dir/www/ddaccordion.js")) {
		show(' * * Creating ddaccordion link...');
		run("cd $gforge_lib_dir/www && ln -s ../lib/external/ddaccordion.js ddaccordion.js", true);
	}
	if (!file_exists("$gforge_lib_dir/www/jquery.js")) {
		show(' * * Creating jquery link...');
		run("cd $gforge_lib_dir/www && ln -s ../lib/external/jquery.js jquery.js", true);
	}
	if (!file_exists("$gforge_lib_dir/www/mktree.js")) {
		show(' * * Creating mktree link...');
		run("cd $gforge_lib_dir/www && ln -s ../lib/external/mktree.js mktree.js", true);
	}
	if (!file_exists("$gforge_lib_dir/www/selectbox.js")) {
		show(' * * Creating selectbox link...');
		run("cd $gforge_lib_dir/www && ln -s ../lib/external/selectbox.js selectbox.js", true);
	}

	$apache_user = $current['apacheUser'];
	$apache_group = $current['apacheGroup'];
	show(' * Checking/Fixing file permissions...');
	run("chmod 755 $gforge_lib_dir/lib/external/viewvc/bin/cgi/viewvc.cgi");
	run('chmod 755 /bin/cvssh.pl');
	exec("find $gforge_lib_dir -name 'bin' -type d", $dirs);
	foreach ($dirs as $dir) {
		if (strtolower(php_uname('s')) == 'sunos') {
		    run("find $dir  -type f -exec chmod +x {} \\;");
		} else {
		    run("find $dir -maxdepth 1 -type f -exec chmod +x {} \\;");
		}
	}
	run("find $gforge_lib_dir -name \"README\" -type f -exec chmod -x {} \\;");

	run("chown -R $apache_user:$apache_group $gforge_lib_dir");
	run("chown -R $apache_user:$apache_group $gforge_etc_dir");
	run("chown -R $apache_user:$apache_group $gforge_var_dir/svnroot");
	run("chown -R $apache_user:$apache_group $gforge_var_dir/caches");
	run("chown -R $apache_user:$apache_group $gforge_var_dir/filesystem");
	
	// Create the exports directory and set its owner properly
	if (!is_dir($gforge_var_dir.'/exportProjects')) {
		mkdir($gforge_var_dir.'/exportProjects');
	}
	run("chown -R $apache_user:$apache_group $gforge_var_dir/exportProjects");

	show(' * Updating GForge DB...');
	run("cd $gforge_lib_dir/db/pgsql && php db-updater.php");
	run("cd $gforge_lib_dir/db/pgsql && php db-updater.php");


	show(' * Updating Config Cache...');
	run("cd $gforge_lib_dir/bin && php create_config_cache.php");
	
	show(' * Updating Language Cache...');
	run("cd $gforge_lib_dir/bin && php create_lang_cache.php");

	echo "\n".RED.'Be sure to read README.PERFORMANCE !'.NORMAL."\n";
	finish();
}

function printVal($val) {
	if (is_array($val)) {
		return 'array('.implode(', ', $val).')';
	} else if (is_bool($val)) {
		if ($val) {
			return 'true';
		} else {
			return 'false';
		}
	} else {
		return $val;
	}
}

function readVal($val, $oldval) {
	$newval = (substr($val, -1) == ';') ? substr($val, 0, -1) : $val;
	if (substr(strtolower($newval), 0, 6) == 'array(' && is_array($oldval)) {
		return $newval;
	} else if (is_numeric($newval) && (is_int($oldval) || is_float($oldval))) {
		return $newval;
	} else if (strtolower($newval)=='true' && is_bool($oldval)) {
		return 'true';
	} else if (strtolower($newval)=='false'&& is_bool($oldval)) {
		return 'false';
	} else {
		return "'$val'";
	}
}

function finish() {
	show("Done.\n");
}

function addToConf($conffile, $key, $oldval, $newval) {
	$configvar = "\$config['$key']=".readVal($newval, $oldval).";";
	$contents = trim(file_get_contents($conffile));
	$contents = substr($contents, 0, -2);
	$contents .= "\n$configvar\n?>";
	file_put_contents($conffile, $contents);
}

function show($text, $newLine = true) {
	if ($newLine) {
		$text = GREEN.$text .NORMAL."\n";
	}
	fwrite(STDOUT, $text);
}

function run($command, $ignore = false) {
	system($command, $ret);
	if ($ret != 0 && $ignore == false) {
		echo RED.'An error ocurred running the last command... aborting.'.NORMAL."\n";
		die();
	}
}

upgrade();
?>
