<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version $Id$
 * @author Daniel A. P�rez	daniel@gforgegroup.com
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

require_once('lib/core/TagEntryPeer.php');

// this file contains common search functions for site and project

	/**
	 * returns the search results for the given input
	 *
	 * @param array $params
	 * @return resultset
	 */
	function getSearchResults($params) {
		$section = $params['section'];
		if (strlen($params['words'])>0) {
			$searcher = new GFSearch($params);
			if ($section=='project') {
				return searchProjects($searcher);
			} elseif ($section=='user') {
				if (isset($params['sub_section']) && $params['sub_section'] == 'addUserSearch') {
					return addUserSearch($searcher);
				} else {
					return searchUsers($searcher);
				}
			}
		} else {
			return "Empty Search";
		}
	}

	/**
	 *	take a search object and handle the search internally.
	 *	@param	GFSearch
	 *	@return string
	 */
	function searchProjects(GFSearch $search) {
		if (GFConf::get('system.dbtype') == 'postgres') {		
			$con = Propel::getConnection(UserPeer::DATABASE_NAME);
			$words = $search->getWordsForQuery();
			
			$columns = array(
			Language::getSessText('SiteAdmin.ProjectList.ProjectName'),
			Language::getSessText('Project.DescriptionName')			
			);
			
			

			if ($words == '*') {
				$sql = "SELECT COUNT(*)
				FROM project
				WHERE is_public=true";
			} else {
				$sql = "SELECT COUNT(*)
				FROM project, projects_idx, to_tsquery('$words') AS q 
				WHERE (project.project_id=projects_idx.project_id 
				AND (projects_idx.vectors @@ q))
				AND project.is_public=true";
			}
			$stmt = $con->createStatement();
			$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_NUM);
			$rs->next();
			$total = $rs->getInt(1);
			
			
			if ($words == '*') {
				$_REQUEST['prjsearchsort_order'] = 'asc';
				$paginator = new Paginator('prjsearch', -1, $total, 'project_id', $columns);
				$sql = "SELECT project_name,description,project_id
				FROM project
				WHERE is_public=true
				" . $paginator->getSQL();				
			} else {
				$_REQUEST['prjsearchsort_order'] = 'desc';
				$paginator = new Paginator('prjsearch', -1, $total, 'rank(projects_idx.vectors, q)', $columns);				
				$sql = "SELECT headline(project.project_name, q) as project_name,
				headline(project.description, q) as description, project.project_id
				FROM project, projects_idx, to_tsquery('$words') AS q 
				WHERE (project.project_id=projects_idx.project_id 
				AND (projects_idx.vectors @@ q))
				AND project.is_public=true
				" . $paginator->getSQL();
			}
			
			$template = $search->getTemplate();

			$ret = '<br />';
			$ret .= $template->tableTop($paginator->getSortableTableHeaders($template));

			$stmt = $con->createStatement();
			$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_ASSOC);
			if ($rs->getRecordCount()>0) {
				$project = ProjectPeer::retrieveByPk($search->getProjectId());
				$i = 0;
				while ($rs->next()) {
					$trow = array();
					$project_id = $rs->getInt('project_id');
					$project = ProjectPeer::retrieveByPk($project_id);
					$trow[] = '<a href="'.GFUrl::getUrl()->projectURL($project) . '">' . $rs->getString('project_name').'</a>';
					$trow[] = $rs->getString('description');
					$ret .= $template->tableRow($trow);
				}
			} else {
				// zero results for search
				$ret .= $template->tableFullRow(Language::getSessText('Search.NoResults'));
			}
			
			$ret .= $template->tableBottom();
			if(isset($paginator) && is_object($paginator)) {
				$ret .= '<div class="paginator">' . $paginator->getPaginator() . '</div><br />';
			}

			return $ret;
		} elseif (GFConf::get('system.dbtype') == 'mysql') {
			
		} elseif (GFConf::get('system.dbtype') == 'oracle') {
			
		}
	}

	/**
	 *	take a search object and handle the search internally.
	 *	@param	GFSearch
	 *	@return string
	 */
	function searchUsers(GFSearch $search) {
		if (GFConf::get('system.dbtype') == 'postgres') {		
			$con = Propel::getConnection(UserPeer::DATABASE_NAME);
			$words = $search->getWordsForQuery();
			$columns = array(
				Language::getSessText('User.UnixNameName'),
				Language::getSessText('SiteAdmin.Search.UserName'));
				
			if ($words == '*') {
				$sql = "SELECT COUNT(*) 
				FROM \"user\"
				WHERE status=". User::STATUS_ACTIVE;
			} else {
				$sql = "SELECT COUNT(*)
				FROM \"user\", users_idx, to_tsquery('$words') AS q 
				WHERE (\"user\".user_id=users_idx.user_id 
				AND (users_idx.vectors @@ q))
				AND \"user\".status=". User::STATUS_ACTIVE;
			}
			
			$stmt = $con->createStatement();
			$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_NUM);
			$rs->next();
			$total = $rs->getInt(1);
			
			if ($words == '*') {
				$paginator = new Paginator('usrsearch', -1, $total, 'user_id', $columns);	
				$sql = "SELECT unix_name,user_id,firstname,lastname 
				FROM \"user\"
				WHERE status=". User::STATUS_ACTIVE . ' ' . $paginator->getSQL();
			} else {
				$_REQUEST['usrsearchsort_order'] = 'desc';
				$paginator = new Paginator('usrsearch', -1, $total, 'rank(users_idx.vectors, q)', $columns);					
				$sql = "SELECT headline(\"user\".unix_name, q) as unix_name, \"user\".user_id,
				headline(\"user\".firstname, q) as firstname, headline(\"user\".lastname, q) as lastname
				FROM \"user\", users_idx, to_tsquery('$words') AS q 
				WHERE (\"user\".user_id=users_idx.user_id 
				AND (users_idx.vectors @@ q))
				AND \"user\".status=". User::STATUS_ACTIVE . ' ' . $paginator->getSQL();
			}
			
			$stmt = $con->createStatement();
			$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_ASSOC);
			
			$template = $search->getTemplate();
			$ret = '<br />';
			$ret .= $template->tableTop($paginator->getSortableTableHeaders($template));
			
			if ($rs->getRecordCount()>0) {
				$project = ProjectPeer::retrieveByPk($search->getProjectId());
				$i = 0;
				while ($rs->next()) {
					$user_id = $rs->getInt('user_id');
					$user = UserPeer::retrieveByPk($user_id);
					$trow = array();
					$trow[] = '<a href="'.GFUrl::getUrl()->userURL($user) . '">' . $rs->getString('unix_name').'</a>';
					$trow[] = $rs->getString('firstname').' '.$rs->getString('lastname');
					$ret .= $template->tableRow($trow);
				}
			} else {
				// zero results for search
				$ret .= $template->tableFullRow(Language::getSessText('Search.NoResults'));
			}
			
			$ret .= $template->tableBottom();
			if(isset($paginator) && is_object($paginator)) {
				$ret .= '<div class="paginator">' . $paginator->getPaginator() . '</div><br />';
			}
			return $ret;
		} elseif (GFConf::get('system.dbtype') == 'mysql') {
			
		} elseif (GFConf::get('system.dbtype') == 'oracle') {
			
		}		
	}	


	function searchTags($search_params){

		if(array_key_exists('project', $search_params)){
			$global_search = false;
		} else {
			$global_search = true;
		}

		$template = $search_params['template'];
		$Lang = Session::getSession()->getLanguage();

		$ret = '<br />';
		$ret = '<p><h3>' . $Lang->getText('Tag.SearchResultTitle') . '</h3></p>';
		$thead = array();
		if($global_search){
			$thead[] = $Lang->getText('Project.ListNameTitle');
		}
		$thead[] = $Lang->getText('Tag.SearchResultRefIdTitle');
		$thead[] = $Lang->getText('Tag.SearchResultSectionTitle');
		$thead[] = $Lang->getText('Tag.SearchResultSubSectionTitle');
		$thead[] = $Lang->getText('Tag.SearchResultDescriptionTitle');
		$ret .= $template->tableTop($thead);

		$results = ActionManager::notify('DO_TAG_SEARCH', $search_params);

		$printed_sections = 0;
		foreach($results as $result){
			$tmp = $result->getResult();
			if(!is_null($tmp)){
				$ret .= $tmp;
				$printed_sections++;
			} 
		}

		if(!$printed_sections){
			$ret .= $template->TableFullRow($Lang->getText('Tag.NoneFound'));
		}

		$ret .=$template->tableBottom();

		return $ret;

	}

	/**
	 *	take a search object and handle the search internally.
	 *	@param	GFSearch
	 *	@return string
	 */
	function addUserSearch(GFSearch $search) {
		if (GFConf::get('system.dbtype') == 'postgres') {		
			$con = Propel::getConnection(UserPeer::DATABASE_NAME);
			$words = $search->getWordsForQuery();
			
			if (isset($search->params['groups']) && !$search->params['groups']) {
				// Don't search in groups
				$groups_sql = 'AND "user".is_group=false';
			} else {
				$groups_sql = '';
			}
			
			if ($words == '*') {
				$sql = "SELECT unix_name,user_id,firstname,lastname 
				FROM \"user\"
				WHERE status=". User::STATUS_ACTIVE ."
				$groups_sql
				ORDER BY user_id LIMIT 250";
			} else {

				$sql = "SELECT headline(\"user\".unix_name, q) as unix_name, \"user\".user_id,
				headline(\"user\".firstname, q) as firstname, headline(\"user\".lastname, q) as lastname
				FROM \"user\", users_idx, to_tsquery('$words') AS q 
				WHERE (\"user\".user_id=users_idx.user_id 
				$groups_sql
				AND (users_idx.vectors @@ q) AND \"user\".status = 1)
				AND \"user\".status=". User::STATUS_ACTIVE ."
				ORDER BY rank(users_idx.vectors, q) DESC";
			}
			$stmt = $con->createStatement();
			$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_ASSOC);
			if ($rs->getRecordCount()>0) {
				$ret = '';
				$template = $search->getTemplate();
				$project = ProjectPeer::retrieveByPk($search->getProjectId());
				$ret = $template->tableTop(array(Language::getSessText('User.UnixNameName'),Language::getSessText('SiteAdmin.Search.UserName')));
				$i = 0;
				while ($rs->next()) {
					$user_id = $rs->getInt('user_id');
					$user = UserPeer::retrieveByPk($user_id);
					$trow = array();
					if (isset($_REQUEST['js_accumulate_users']) && !empty($_REQUEST['js_accumulate_users'])) {
						$trow[] = '<a href="#" onclick="if(window.opener.document.useradd.share_users.value.length > 0) { window.opener.document.useradd.share_users.value = window.opener.document.useradd.share_users.value + \','.$user->getUnixName().'\'; } else { window.opener.document.useradd.share_users.value = \''.$user->getUnixName().'\'; } window.close();">' . $rs->getString('unix_name').'</a>';
					} else {
						$trow[] = '<a href="#" onclick="window.opener.document.useradd.unix_name.value = \''.$user->getUnixName().'\'; window.close();">' . $rs->getString('unix_name').'</a>';
					}
					$trow[] = $rs->getString('firstname').' '.$rs->getString('lastname');
					$ret .= $template->tableRow($trow);
				}
				$ret .= $template->tableBottom();
				return $ret;
			} else {
				// cero results for search
				return Language::getSessText('Search.NoResults');
			}
		} elseif (GFConf::get('system.dbtype') == 'mysql') {
			
		} elseif (GFConf::get('system.dbtype') == 'oracle') {
			
		}		
	}	
